require_dependency "skirental_payment/application_controller"
require 'paypal-sdk-rest'
include PayPal::SDK::REST
include PayPal::SDK::Core::Logging

module SkirentalPayment
    class PaypalPaymentsController < PaymentsController

        def create
            booking = SkirentalFrontendApi::Booking.where(shop_id: paypal_payment_params[:shop_id], session_id: paypal_payment_params[:session_id]).first
            fail if booking.nil?

            fail if paypal_payment_params[:email].blank?
            save_email(booking, paypal_payment_params[:email])

            config = Rails.application.config.payments[:PAYPAL]

            shop_settings = SkirentalApi::Client.settings(paypal_payment_params[:shop_id])

            transaction = booking.transaction(shop_settings[:money][:tax][:include], shop_settings[:money][:tax][:show], 'paypal')

            if config[:mode] == "sandbox"
                paypal_ids = config
            else
                paypal_ids = SkirentalApi::Client.paypal_credentials(paypal_payment_params[:shop_id])
                fail "No PayPal-Data available for shop: #{shop_id}" if paypal_ids.nil?
            end

            PayPal::SDK::REST.set_config(
                :mode           => config[:mode], # "sandbox" or "live"
                :client_id      => paypal_ids[:client_id],
                :client_secret  => paypal_ids[:client_secret],
                # Deliberately set ca_file to nil so the system's Cert Authority is used,
                # instead of the bundled paypal.crt file which is out-of-date due to:
                # https://www.paypal.com/va/smarthelp/article/discontinue-use-of-verisign-g5-root-certificates-ts2240
                :ssl_options    => {
                    :ca_file => nil
                }
            )

            payment_context = SkirentalPayment::Payment.new(booking_id: booking.id, is_payed: false, transaction: transaction)

            # Build Payment object
            @payment = PayPal::SDK::REST::Payment.new({
                                                        :intent =>  "sale",
                                                        :payer =>  {
                                                            :payment_method =>  "paypal"
                                                        },
                                                        :redirect_urls => {
                                                            :return_url => "#{request.protocol}#{request.host_with_port}",
                                                            :cancel_url => "#{request.protocol}#{request.host_with_port}"
                                                        },
                                                        :transactions => [transaction]
            })

            if @payment.create
                payment_context.payment_reference = @payment.id # Payment Id
                payment_context.provider_name = "paypal"

                if payment_context.save
                    render json: { payment: @payment }, status: :created
                else
                    render json: { error: payment_context.errors.full_messages }, status: :internal_server_error
                end
            else
                @payment.error  # Error Hash
                render json: { error: @payment.error  }, status: :bad_request
            end

        end

        # Is called by Client (Paypal-Checkout-Flow Window) after payer presses "Pay Now"
        def execute
            @payment = PayPal::SDK::REST::Payment.find(paypal_payment_params[:payment_id])

            # PayerID is required to approve the payment.
            if @payment.execute( :payer_id => paypal_payment_params[:payer_id] )
                logger.info "Payment[#{@payment.id}] execute successfully"

                payment_context = SkirentalPayment::Payment.where(payment_reference: @payment.id).first
                fail if payment_context.nil?

                payment_context.is_payed = true
                payment_context.provider_name = "Paypal"
                confirm_payment(payment_context, false)
                if payment_context.save
                    render json: { payment: @payment, voucher: payment_context.voucher_code  }, status: :ok
                else
                    render json: { error: payment_context.errors.full_messages }, status: :internal_server_error
                end
            else
                logger.error @payment.error.inspect
                render json: { error: @payment.error.inspect }, status: :bad_request
            end
        end


        # is called by client to fetch the backends paypal mode (sandbox|live)
        def environment
            render json: { mode: Rails.application.config.payments[:PAYPAL][:mode] }
        end

        private

        def save_email(booking, email)
            booking[:context][:customers].first[:personalData][:address][:email] = email
            booking.save
        end

        def paypal_payment_params
            params.permit(:shop_id, :session_id, :payment_id, :payer_id, :email)
        end
    end
end
