require_dependency "skirental_payment/application_controller"
require "stripe"

module SkirentalPayment
    class StripePaymentsController < PaymentsController

        def create
            shop_id = stripe_payment_params[:shop_id]
            session_id = stripe_payment_params[:session_id]
            email = stripe_payment_params[:email]
            connected_account_id = stripe_payment_params[:connected_account_id]
            locale = stripe_payment_params[:locale]
            fail "No shop id provided" if shop_id.blank?
            fail "No session id provided" if session_id.blank?
            fail "No email address provided" if email.blank?
            fail "No connected account id for stripe provided" if connected_account_id.blank?

            booking = SkirentalFrontendApi::Booking.where(shop_id: shop_id, session_id: session_id).first
            fail "No booking found for given shop & session" if booking.nil?

            Stripe.api_key = Rails.application.config.payments[:STRIPE][:secret_key]

            save_email(booking, email)

            shop_settings = SkirentalApi::Client.settings(shop_id)
            transaction = booking.transaction(true, false, 'stripe')  # NOTE: stripe expects gross prices only
            payment_context = SkirentalPayment::Payment.new(booking_id: booking.id, is_payed: false, transaction: transaction)

            application_fee_amount = 0
            shop_settings[:money][:payment][:ipMs].each do |payment_provider|
                if payment_provider[:type] == 'stripe'
                    application_fee_amount = (payment_provider[:fee].to_d * 100).round  # WARNING: stripe wants this to be cents…
                    break
                end
            end

            begin
                base_url = Rails.application.config.frontend_api[:base_url] + (locale.blank? ? "" : "/#{locale}")
                base_url = "#{base_url}/#{shop_id}/reservation"
                stripe_locales = ["da", "de", "en", "es", "fi", "fr", "it", "ja", "nb", "nl", "pl", "pt", "sv", "zh"]
                unless locale.blank? or stripe_locales.exclude?(locale)
                    locale = "auto"
                end
                checkout_session = Stripe::Checkout::Session.create({  # https://stripe.com/docs/api/checkout/sessions/create
                    success_url: "#{base_url}/checkout-awaiting-confirmation?session=#{session_id}",
                    cancel_url: "#{base_url}/checkout-payment?session=#{session_id}",
                    payment_method_types: ["card"],
                    client_reference_id: "#{shop_id}|#{session_id}",
                    customer_email: email,
                    line_items: transaction[:item_list][:items].map { |item|
                        {
                            amount:      (item[:price].to_d * 100).round,  # WARNING: stripe wants this to be cents…
                            currency:    item[:currency],
                            name:        item[:name],
                            quantity:    item[:quantity],
                            description: item[:description].blank? ? nil : item[:description],
                            #images: ["A list of images representing this line item."]
                        }
                    },
                    locale: locale.blank? ? "auto" : locale,
                    payment_intent_data: {
                        application_fee_amount: application_fee_amount
                    },
                }, { stripe_account: connected_account_id })

                @checkout_session_id = checkout_session.id
                puts "CREATED STRIPE CHECKOUT SESSION WITH ID: #{@checkout_session_id}"

                payment_context.payment_reference = @checkout_session_id  # checkout session Id
                payment_context.provider_name = "stripe"

                if payment_context.save
                    render json: { checkout_session_id: @checkout_session_id }, status: :created
                else
                    render json: { error: payment_context.errors.full_messages }, status: :internal_server_error
                end
            rescue => error
                render json: { error: error  }, status: :bad_request
            end
        end

        def check
            checkout_session_id = stripe_payment_params[:checkout_session_id]
            fail "No checkout session id provided" if checkout_session_id.blank?

            payment = SkirentalPayment::Payment.where(payment_reference: checkout_session_id).first
            if payment.nil?
                render json: { is_payed: false, payment_found: false }
                return
            end

            render status: 200, json: { is_payed: payment.is_payed, payment_found: true, voucher: payment.voucher_code }
        end

        def webhook
            # You can use webhooks to receive information about asynchronous payment events.
            # For more about our webhook events check out https://stripe.com/docs/webhooks.

            webhook_secret = Rails.application.config.payments[:STRIPE][:webhook_secret]
            event = JSON.parse(request.body.read)

            puts "PROCESSING STRIPE WEBHOOK WITH:"
            puts "#{event}"

            if webhook_secret
                # Retrieve the event by verifying the signature using the raw body and secret if webhook signing is configured.
                sig_header = request.env['HTTP_STRIPE_SIGNATURE']

                begin
                    puts "STRIPE WEBHOOK: trying to run Stripe::Webhook.construct_event with: "
                    puts "sig_header: #{sig_header}"
                    puts "webhok_secret: #{webhook_secret}"
                    puts "request.body:"
                    puts "#{request.body.read}"
                    event = Stripe::Webhook.construct_event(
                        request.body.read, sig_header, webhook_secret
                    )
                rescue JSON::ParserError => e
                    puts "STRIPE WEBHOOK: INVALID PAYLOAD!"
                    # Invalid payload
                    render status: 400, json: {
                        error: 'invalid payload'
                    }
                    return
                rescue Stripe::SignatureVerificationError => e
                    puts "STRIPE WEBHOOK: INVALID SIGNATURE!"
                    puts "#{e}"
                    # Invalid signature
                    render status: 400, json: {
                        error: 'invalid signature'
                    }
                    return
                end

                if event['type'] == 'checkout.session.completed'
                    session = event['data']['object']
                    client_reference_id = session['client_reference_id']
                    puts "🔔  Webhook received! Checkout for #{client_reference_id} completed"

                    # Fulfill the purchase using the client_reference_id...
                    checkout_session_id = session['id']
                    payment_context = SkirentalPayment::Payment.where(payment_reference: checkout_session_id).first
                    if payment_context.nil?
                        puts "STRIPE WEBHOOK: PAYMENT NOT FOUND!"
                        puts "checkout session #{checkout_session_id}"
                        render status: 400, json: {
                            error: 'payment not found'
                        }
                        return
                    end
                    logger.info "Payment[#{checkout_session_id}] executed successfully"

                    payment_context.is_payed = true
                    payment_context.provider_name = "Stripe"
                    confirm_payment(payment_context, false)

                    if payment_context.save
                        puts "STRIPE WEBHOOK: payment updated successfully"
                        render status: 200, json: {
                            status: 'payment context successfully updated'
                        }
                    else
                        puts "STRIPE WEBHOOK: could not update payment context"
                        puts "#{payment_context.errors.full_messages}"
                        render json: { error: payment_context.errors.full_messages }, status: :internal_server_error
                    end

                    return
                else
                    puts "STRIPE WEBHOOK: INVALID EVENT TYPE!"
                    puts "#{event['type']}"
                    render status: 400, json: {
                        error: 'invalid event type'
                    }
                end
            else
                puts "STRIPE WEBHOOK: NO SECRET DEFINED IN CONFIG!"
                render status: 400, json: {
                    error: 'no webhook_secret defined'
                }
            end
        end

        # is called by client to fetch the backends stripe mode (test|live)
        def environment
            render json: { mode: Rails.application.config.payments[:stripe][:mode] }
        end

        private

        def save_email(booking, email)
            booking[:context][:customers].first[:personalData][:address][:email] = email
            booking.save
        end

        def stripe_payment_params
            params.permit(:shop_id, :session_id, :connected_account_id, :email, :locale, :checkout_session_id)
        end
    end
end
